@@ -1,9 +1,10 @@
-import traceback
+import traceback, json
 
 from functools  import wraps
 from colored    import fg, attr
 from shutil     import rmtree
-from time   import time
+from typing     import Any
+from time   import perf_counter
 from os     import mkdir, listdir, remove
 
 def try_traceback(skip_traceback=False):
@@ -25,9 +26,9 @@ def func_timer(ret_time=False, do_print=True):
         @wraps(func)
         def wrapper(*args, **kwargs):
             fprint('Function timer started', do_print=do_print)
-            time_start = time()
+            time_start = perf_counter()
             ret_val = func(*args, **kwargs)
-            time_delta = time() - time_start
+            time_delta = perf_counter() - time_start
             fprint(f'Function {func.__name__}{args} {kwargs} Took {time_delta:.4f} seconds to run', do_print=do_print)
             if ret_time:
                 return ret_val, time_delta
@@ -36,7 +37,7 @@ def func_timer(ret_time=False, do_print=True):
     return timer
 
 
-def fprint(text=None, nl=True, flush=False, do_print=True):
+def fprint(text=None, nl=True, flush=False, do_print=True, end='') -> None:
     if not text:
         text = ''
 
@@ -44,7 +45,10 @@ def fprint(text=None, nl=True, flush=False, do_print=True):
         text = f'{text}\n'
 
     if do_print:
-        print(text, flush=flush)
+        if not end:
+            print(text, flush=flush)
+        else:
+            print(text, end=end, flush=flush)
 
 
 def finput(text='', nl=True, use_end_addon=True, ret_type: type = str):
@@ -64,10 +68,10 @@ def finput(text='', nl=True, use_end_addon=True, ret_type: type = str):
         print(f'Couldn\'t return input {inpt} as {ret_type}')
         print(f'Input type: {type(inpt)}')
         fprint('Returning value as string')
-        return inpt
+    return inpt
 
 
-def enter_to_continue(text='', nl=True, use_help_text=True):
+def enter_to_continue(text='', nl=True, use_help_text=True) -> bool:
     if text and use_help_text:
         text = f'{text} (press enter to continue) '
 
@@ -115,32 +119,34 @@ def choose_from_list(lst, header_text='', header_nl=False, input_text='Input ind
             return []
 
 
-def read_file(file_path, create=False, ret_did_create=False, remove_empty=True, do_splitlines=True, do_print=True, encoding='utf-8'):
-    def try_read(did_create):
+def read_file(file_path, create:bool=False, ret_did_create:bool=False, remove_empty:bool=True, splitlines:bool=True, do_print:bool=True, encoding:str='utf-8', strip:bool=True) -> Any:
+    def try_reading(did_create:bool=False):
+        lines: list[str] | str = []
         try:
             with open(file_path, 'r', encoding=encoding) as f:
                 lines = f.read()
-                if do_splitlines:
-                    lines = lines.splitlines()
+            if splitlines:
+                lines = lines.splitlines()
+                if remove_empty:
+                    lines = [x for x in lines if x != '']
+                if strip:
+                    lines = [x.strip() for x in lines]
             return lines, did_create
         except FileNotFoundError:
+            fprint(f'File not found: {file_path}')
             if create:
-                create_file_dir('file', file_path, force=True)
-                did_create = True
-                return try_read(did_create)
-            else:
-                fprint(f'File not found: {file_path}', do_print=do_print) 
-    lines, did_create = try_read(False) #type:ignore
+                create_file_dir('f', file_path, force=True, do_print=do_print)
+                return try_reading(True)
+        return lines, did_create
 
-    if remove_empty and lines and do_splitlines:
-        lines = [x for x in lines if x != '']
+    lines,did_create = try_reading()
 
     if ret_did_create:
-        return lines, did_create
+        return lines,did_create
     return lines
 
 
-def write_file(text, file_path, append=False, create=True, encoding='utf-8', do_print=True):
+def write_file(text, file_path, append=False, create=True, encoding='utf-8', do_print=True) -> bool:
     if text.__class__.__name__ in ('list', 'tuple', 'set'):
         text = '\n'.join(text)
 
@@ -173,11 +179,11 @@ def create_file_dir(do, do_path, force=False, do_print=True):
                 fprint(f'Directory removed: {do_path}', nl=False, do_print=do_print)
                 return create_dir()
 
-    if do == 'dir':
+    if do == 'd': # Directory
         create_dir()
     
-    elif do == 'file':
-        file_dir, filename = get_dir_path_for_file(do_path) #type:ignore
+    elif do == 'f': # File
+        file_dir, filename = get_dir_path_for_file(do_path)
         files_in_path = listdir(file_dir)
 
         if force or filename not in files_in_path:
@@ -192,7 +198,7 @@ def create_file_dir(do, do_path, force=False, do_print=True):
 
 @try_traceback(skip_traceback=True)
 def remove_file_dir(do, do_path, force=False, do_print=True):
-    if do == 'dir':
+    if do == 'd': # Directory
         try:
             dir_content = listdir(do_path)
         except FileNotFoundError:
@@ -206,7 +212,7 @@ def remove_file_dir(do, do_path, force=False, do_print=True):
         elif dir_content:
             fprint(f'Directory is not empty, not removing: {do_path}', do_print=do_print)
 
-    elif do == 'file':
+    elif do == 'f': # File
         lines = read_file(do_path)
         if lines and not force:
             fprint(f'File is not empty, not removing: {do_path}', do_print=do_print)
@@ -215,7 +221,7 @@ def remove_file_dir(do, do_path, force=False, do_print=True):
         fprint(f'File removed: {do_path}', do_print=do_print)
 
 
-def get_dir_path_for_file(file_path, ret_val='all'):
+def get_dir_path_for_file(file_path, ret_val='a') -> Any:
     dir_path = file_path.replace('\\', '/').split('/')
     if len(dir_path) == 1:
         dir_path = None
@@ -224,15 +230,15 @@ def get_dir_path_for_file(file_path, ret_val='all'):
         filename = dir_path.pop()
         dir_path = '/'.join(dir_path)
 
-    if ret_val == 'dir':
+    if ret_val == 'd': # Directory
         return dir_path
-    elif ret_val == 'filename':
+    elif ret_val == 'fnam': # Filename
         return filename
 
     return dir_path, filename
 
 
-def fd(variable, nl=False):
+def fd(variable, nl=False) -> None:
     reset = attr('reset')
     filename, lineno, function_name, code = traceback.extract_stack()[-2]
     original_var = code.replace('fd(', '').replace(')', '', 1)
@@ -254,4 +260,24 @@ def fd(variable, nl=False):
 
 
 def join_path(*args, join_with='\\'):
-    return join_with.join(args)
\ No newline at end of file
+    return join_with.join(args)
+
+
+def create_json(filepath:str, force:bool=False, do_print:bool=True) -> bool:
+    did_create = create_file_dir('f', filepath, force=force, do_print=do_print)
+    if did_create:
+        open(filepath, 'w').write(json.dumps({}, indent=4))
+    return did_create
+
+
+def read_json(filepath:str):
+    try:
+        with open(filepath, 'r') as f:
+            file_data = json.load(f)
+        return file_data
+    except FileNotFoundError:
+        fprint(f'File not found: {filepath}')
+    except json.decoder.JSONDecodeError:
+        fprint(f'File cannot be read as a JSON: {filepath}')
+    except Exception:
+        fprint(traceback.format_exc())
\ No newline at end of file
